# Location Class

from _globals import *
from Classes.class_dc import DC
from Classes.class_eud_remastered import EUD_Remastered
from Classes.class_location import Location
from inspect import isfunction
import Triggers

class EUD_Location(Location):
    def __init__(self, name, loc_num, virtual_location = None, v_loc = None):
        #loc_num here is the index of the location.  e.g. Location 1 really has index 0.
        self.x_min_address = 3647+((loc_num)*5)
        self.x_min = EUD_Remastered(self.x_min_address,0)
        self.y_min_address = 3648+((loc_num)*5)
        self.y_min = EUD_Remastered(self.y_min_address,0)
        self.x_max_address = 3649+((loc_num)*5)
        self.x_max = EUD_Remastered(self.x_max_address,0)
        self.y_max_address = 3650+((loc_num)*5)
        self.y_max = EUD_Remastered(self.y_max_address,0)
        
        if(not (virtual_location == None) ):
            self.is_virtual = True
        else:
            self.is_virtual = False
        
        #This is the array containing x_min,x_max,y_min,y_max
        self.virtual_location = virtual_location
        if self.is_virtual:
            self.v_loc = EUD_Location(name,loc_num)
        
        
        if isinstance(name, str):
            self.name = name
        elif isinstance(name, int):
            self.name = "Location "+str(name)
        
            
    # Conditions
    def bring(self, player = "Current Player", unit = "Any unit", qmod = "At least", num = 1):
        if(self.is_virtual):
            out = [
                Triggers.Trigger.T(  
                    A=[
                        self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        self.v_loc.remove_unit(player,"Ruins")
                ] )
            ]
            out.append(
                Triggers.Trigger.T(  
                    C=[self.v_loc.bring(player, unit, qmod, num)],
                    A=[True]
                )
            )
            return out
        else:
            return Location.bring(self,player,unit,qmod,num)
        
    def bring_none(self, player = "Current Player", unit = "Any unit", qmod = "Exactly", num = 0):
        if(self.is_virtual):
            out = [
                Triggers.Trigger.T(  
                    A=[
                        self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        self.v_loc.remove_unit(cp,"Ruins")
                ] )
            ]
            out.append(
                Triggers.Trigger.T(  
                    C=[self.v_loc.bring(player, unit, qmod, num)],
                    A=[True]
                )
            )
            return out
        else:
            return Location.bring_none(self,player,unit,qmod,num)
    
    # Actions
    def center_view(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        Triggers.Actions.center_view(self.name)]
        else:
            return  Triggers.Actions.center_view(self.name)
    
    def create_unit(self, player, unit, num = 1, property = None):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        Triggers.Actions.create_unit(player, unit, num, self.name, property)]
        else:
            return  Triggers.Actions.create_unit(player, unit, num, self.name, property)
        
    def give_unit(self, old_player, new_player, unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        Triggers.Actions.give_unit(old_player, new_player, unit, num, self.name)]
        else:
            return  Triggers.Actions.give_unit(old_player, new_player, unit, num, self.name)
        
    def kill_unit(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        Triggers.Actions.kill_unit_at_location(player, unit, num, self.name)]
        else:
            return  Triggers.Actions.kill_unit_at_location(player, unit, num, self.name)
        
    def set_energy(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_energy(player, unit, amount, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_energy(player, unit, amount, num, self.name)
        
    def max_energy(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_energy(player, unit, 100, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_energy(player, unit, 100, num, self.name)
        
    def kill_energy(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_energy(player, unit, 0, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_energy(player, unit, 0, num, self.name)
        
    def set_hanger_count(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, amount, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, amount, num, self.name)
        
    def max_hanger_count(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, 200, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, 200, num, self.name)
        
    def kill_hanger_count(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, 0, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_hanger_counTriggers.Trigger.T(player, unit, 0, num, self.name)
        
    def set_hit_points(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_hit_points(player, unit, amount, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_hit_points(player, unit, amount, num, self.name)
        
    def max_hit_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_hit_points(player, unit, 100, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_hit_points(player, unit, 100, num, self.name)
        
    def set_shield_points(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_shield_points(player, unit, amount, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_shield_points(player, unit, amount, num, self.name)
        
    def max_shield_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_shield_points(player, unit, 100, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_shield_points(player, unit, 100, num, self.name)
        
    def kill_shield_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_shield_points(player, unit, 0, num, self.name)]
        else:
            return  Triggers.Actions.modify_unit_shield_points(player, unit, 0, num, self.name)
        
    def set_resource_amount(self, amount, player = "Player 12", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.modify_unit_resource_amounTriggers.Trigger.T(player, amount, num, self.name)   ]
        else:
            return   Triggers.Actions.modify_unit_resource_amounTriggers.Trigger.T(player, amount, num, self.name) 
        
    def move(self, player, unit = None, destination = "Anywhere"):
        out = []
        if(self.is_virtual):
            out.append(self.v_loc.x_min.set_to(self.virtual_location[0]))
            out.append(self.v_loc.x_max.set_to(self.virtual_location[1]))
            out.append(self.v_loc.y_min.set_to(self.virtual_location[2]))
            out.append(self.v_loc.y_max.set_to(self.virtual_location[3]))
        if isinstance(player, Location):
            if isinstance(player, EUD_Location):
                if player.is_virtual:
                    out.append(player.v_loc.x_min.set_to(player.virtual_location[0]))
                    out.append(player.v_loc.x_max.set_to(player.virtual_location[1]))
                    out.append(player.v_loc.y_min.set_to(player.virtual_location[2]))
                    out.append(player.v_loc.y_max.set_to(player.virtual_location[3]))
            out.append(Triggers.Actions.move_location("Player 12", "Map Revealer", player.name, self.name))
            return out
        else:
            if isinstance(destination, EUD_Location):
                if destination.is_virtual:
                    out.append(destination.v_loc.x_min.set_to(destination.virtual_location[0]))
                    out.append(destination.v_loc.x_max.set_to(destination.virtual_location[1]))
                    out.append(destination.v_loc.y_min.set_to(destination.virtual_location[2]))
                    out.append(destination.v_loc.y_max.set_to(destination.virtual_location[3]))
            out.append(Triggers.Actions.move_location(player, unit, destination, self.name))
        return out
        
    def move_unit(self, player, destination, unit = "Any unit", num = "All"):
        out = []
        if(self.is_virtual):
            out.append(self.v_loc.x_min.set_to(self.virtual_location[0]))
            out.append(self.v_loc.x_max.set_to(self.virtual_location[1]))
            out.append(self.v_loc.y_min.set_to(self.virtual_location[2]))
            out.append(self.v_loc.y_max.set_to(self.virtual_location[3]))
        if isinstance(destination, EUD_Location):
            if destination.is_virtual:
                out.append(destination.v_loc.x_min.set_to(destination.virtual_location[0]))
                out.append(destination.v_loc.x_max.set_to(destination.virtual_location[1]))
                out.append(destination.v_loc.y_min.set_to(destination.virtual_location[2]))
                out.append(destination.v_loc.y_max.set_to(destination.virtual_location[3]))
        out.append(Triggers.Actions.move_unit(player, unit, num, self.name, destination))
        return out
        
    def order_attack_to(self, player, destination, unit = "Any unit"):
        out = []
        if(self.is_virtual):
            out.append(self.v_loc.x_min.set_to(self.virtual_location[0]))
            out.append(self.v_loc.x_max.set_to(self.virtual_location[1]))
            out.append(self.v_loc.y_min.set_to(self.virtual_location[2]))
            out.append(self.v_loc.y_max.set_to(self.virtual_location[3]))
        if isinstance(destination, EUD_Location):
            if destination.is_virtual:
                out.append(destination.v_loc.x_min.set_to(destination.virtual_location[0]))
                out.append(destination.v_loc.x_max.set_to(destination.virtual_location[1]))
                out.append(destination.v_loc.y_min.set_to(destination.virtual_location[2]))
                out.append(destination.v_loc.y_max.set_to(destination.virtual_location[3]))
        out.append(Triggers.Actions.order(player, unit, self.name, destination, "Attack"))
        return out
        
    def order_move_to(self, player, destination, unit = "Any unit"):
        out = []
        if(self.is_virtual):
            out.append(self.v_loc.x_min.set_to(self.virtual_location[0]))
            out.append(self.v_loc.x_max.set_to(self.virtual_location[1]))
            out.append(self.v_loc.y_min.set_to(self.virtual_location[2]))
            out.append(self.v_loc.y_max.set_to(self.virtual_location[3]))
        if isinstance(destination, EUD_Location):
            if destination.is_virtual:
                out.append(destination.v_loc.x_min.set_to(destination.virtual_location[0]))
                out.append(destination.v_loc.x_max.set_to(destination.virtual_location[1]))
                out.append(destination.v_loc.y_min.set_to(destination.virtual_location[2]))
                out.append(destination.v_loc.y_max.set_to(destination.virtual_location[3]))
        out.append(Triggers.Actions.order(player, unit, self.name, destination, "Move"))
        return out
        
    def order_patrol_to(self, player, destination, unit = "Any unit"):
        out = []
        if(self.is_virtual):
            out.append(self.v_loc.x_min.set_to(self.virtual_location[0]))
            out.append(self.v_loc.x_max.set_to(self.virtual_location[1]))
            out.append(self.v_loc.y_min.set_to(self.virtual_location[2]))
            out.append(self.v_loc.y_max.set_to(self.virtual_location[3]))
        if isinstance(destination, EUD_Location):
            if destination.is_virtual:
                out.append(destination.v_loc.x_min.set_to(destination.virtual_location[0]))
                out.append(destination.v_loc.x_max.set_to(destination.virtual_location[1]))
                out.append(destination.v_loc.y_min.set_to(destination.virtual_location[2]))
                out.append(destination.v_loc.y_max.set_to(destination.virtual_location[3]))
        out.append(Triggers.Actions.order(player, unit, self.name, destination, "Patrol"))
        return out
        
    def remove_unit(self, player = "Current Player", unit = "Any unit", num = "All"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.remove_unit_at_location(player, unit, num, self.name)]
        else:
            return  Triggers.Actions.remove_unit_at_location(player, unit, num, self.name)
        
    def ai_script(self, type):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location(type, self.name)]
        else:
            return   Triggers.Actions.run_ai_script_at_location(type, self.name)
        
    def set_doodad_state(self, player = "Current Player", unit = "Any unit", state = "enabled"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.set_doodad_state(player, unit, self.name, state)]
        else:
            return  Triggers.Actions.set_doodad_state(player, unit, self.name, state)
        
    def set_invincibility(self, player = "Current Player", unit = "Any unit", state = "enabled"):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.set_invincibility(player, unit, self.name, state)    ]
        else:
            return   Triggers.Actions.set_invincibility(player, unit, self.name, state)
        
    def enter_transport(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location("EnTr", self.name)]
        else:
            return  Triggers.Actions.run_ai_script_at_location("EnTr", self.name)
        
    def exit_transport(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location("ExTr", self.name)    ]
        else:
            return  Triggers.Actions.run_ai_script_at_location("ExTr", self.name)  
        
    def junk_yard_dog(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location("JYDg", self.name)]
        else:
            return   Triggers.Actions.run_ai_script_at_location("JYDg", self.name)
        
    def disruption_web(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location("CWHe", self.name)]
        else:
            return   Triggers.Actions.run_ai_script_at_location("CWHe", self.name)
        
    def recall(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.run_ai_script_at_location("ReHe", self.name)    ]
        else:
            return  Triggers.Actions.run_ai_script_at_location("ReHe", self.name)
        
    def ping(self):
        if(self.is_virtual):
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                         Triggers.Actions.minimap_ping(self.name)]
        else:
            return   Triggers.Actions.minimap_ping(self.name)
        
   
    #More actions
    def align(self):
        if self.is_virtual:
            return [    self.v_loc.x_min.set_to(self.virtual_location[0]),
                        self.v_loc.x_max.set_to(self.virtual_location[1]),
                        self.v_loc.y_min.set_to(self.virtual_location[2]),
                        self.v_loc.y_max.set_to(self.virtual_location[3]),
                        self.v_loc.remove_unit(cp,"Ruins") ]
                        
    def invert_align(self):
        if self.is_virtual:
            return [    self.v_loc.x_min.set_to(self.virtual_location[1]),
                        self.v_loc.x_max.set_to(self.virtual_location[0]),
                        self.v_loc.y_min.set_to(self.virtual_location[3]),
                        self.v_loc.y_max.set_to(self.virtual_location[2]),
                        self.v_loc.remove_unit(cp,"Ruins") ]
    
    def eud_move(self, num1,num2,num3 = None,num4 = None):
        if(not isinstance(num3, int)):
            x_min = num1
            x_max = num1
            y_min = num2
            y_max = num2
        else:
            x_min = num1
            x_max = num2
            y_min = num3
            y_max = num4
        
        out = [self.x_min.set_to(x_min)]
        out.append(self.x_max.set_to(x_max))
        out.append(self.y_min.set_to(y_min))
        out.append(self.y_max.set_to(y_max))
        
        return out
        
    def eud_move_add(self, num1,num2,num3 = None,num4 = None):
        if(not isinstance(num3, int)):
            x_min = num1
            x_max = num1
            y_min = num2
            y_max = num2
        else:
            x_min = num1
            x_max = num2
            y_min = num3
            y_max = num4
        
        if isinstance(num1,DC):
            out = [ self.x_min.add(num1),
                    self.x_max.add(num1),
                    self.y_min.add(num2),
                    self.y_max.add(num2)
                    ]
        else:
            if x_min >= 0:
                out = [self.x_min.add(x_min)]
            else:
                out = [self.x_min.sub(-x_min)]
            
            if x_max >= 0:
                out.append(self.x_max.add(x_max))
            else:
                out.append(self.x_max.sub(-x_max))
                
            if y_min >= 0:
                out.append(self.y_min.add(y_min))
            else:
                out.append(self.y_min.sub(-y_min))
                
            if y_max >= 0:
                out.append(self.y_max.add(y_max))
            else:
                out.append(self.y_max.sub(-y_max))
        
        return out
    
                

        
        
        
        
        
        
        
        
        
        
        