# Location Class
import sys

from _globals import *
import Triggers
from Classes.class_dc import DC
from Classes.class_tempdc import TempDC

resource_max = (2**31)-1

class EUD_Remastered(DC):
    def __init__(self, index, offset, bit_mask = None, mult = None):
        self.index = index
        self.standard = True
        self.masked = False
        self.offset = offset
        self.offset_num = 1
        if offset == 0:
            self.offset_num = 1
        elif offset == 1:
            self.offset_num = 256
        elif offset == 2:
            self.offset_num = 65536
        elif offset == 3:
            self.offset_num = 16777216
        else:
            self.base_address = offset
            self.mult = mult
            self.standard = False
            
        if isinstance(bit_mask, int):
            self.bit_mask = bit_mask
            self.masked = True
        
        self.max = 256*self.offset_num
        
    # Conditions
    
    def __eq__(self, var):
        "overloads == operator, letting you do eud_object == 7, or eud_object == dc2"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Conditions.memory(self.index,"Exactly",var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0, var*self.offset_num == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Conditions.masked_memory(self.index,"Exactly",var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0, var*self.offset_num == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Triggers.Conditions.memory(self.index,"Exactly",self.base_address-(var*self.mult))
    
    def __ge__(self, var):
        "overloads >= operator"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.always()
                    return Triggers.Conditions.memory(self.index,"At least",var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[var*self.offset_num == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.always()
                    return Triggers.Conditions.masked_memory(self.index,"At least",var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[var*self.offset_num == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Triggers.Conditions.memory(self.index,"At most",self.base_address-(var*self.mult))
    
    def geq(self,var,player = None):
        return self >= var
    
    def __gt__(self, var):
        "overloads > operator"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Conditions.memory(self.index,"At least",var*self.offset_num+1)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[var*self.offset_num == 0, self > 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Conditions.masked_memory(self.index,"At least",var*self.offset_num+1,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[var*self.offset_num == 0, self > 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Trigger.Conditions.memory(self.index,"At most",self.base_address-((var+1)*self.mult))
            
    def __le__(self, var):
        "overloads <= operator"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.memory(self.index,"Exactly",0)
                    return Triggers.Conditions.memory(self.index,"At most",var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.masked_memory(self.index,"Exactly",0,self.bit_mask)
                    return Triggers.Conditions.masked_memory(self.index,"At most",var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Trigger.Conditions.memory(self.index,"At least",self.base_address-(var*self.mult))
            
    def __lt__(self, var):
        "overloads < operator"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.never()
                    return Triggers.Conditions.memory(self.index,"At most",var*self.offset_num-1)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0, var*self.offset_num > 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    if var*self.offset_num == 0:
                        return Triggers.Conditions.never()
                    return Triggers.Conditions.masked_memory(self.index,"At most",var*self.offset_num-1,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i, self>=2**i], A=[var*self.offset_num-(2**i), self-(2**i), temp+(2**i)]) )
                    out.append( Triggers.Trigger.T( C=[self == 0, var*self.offset_num > 0], A=[True] ) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i, self+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Trigger.Conditions.memory(self.index,"At least",self.base_address-((var-1)*self.mult))
        
    #Actions
    def set_to(self, var):
        "Sets the DC to its argument.  This of this like an = operator."
        "e.g. dc1.set_to(5), or dc1.set_to(dc2)"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Set To", var*self.offset_num)
                elif isinstance(var, DC):
                
                    if var.max*self.offset_num > self.max:
                        print ("You just tried to set a DC past its max value.")
                        sys.exit()
                
                    max_power = ((var.max*self.offset_num).bit_length()-1)
                    out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+self.offset_num*(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Set To", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                
                    if var.max*self.offset_num > self.max:
                        print ("You just tried to set a DC past its max value.")
                        sys.exit()
                
                    max_power = ((var.max*self.offset_num).bit_length()-1)
                    out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+self.offset_num*(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Triggers.Actions.memory_write(self.index, "Set To", self.base_address-(var*self.mult))
            
    def become(self, var):
        "Sets the DC to its argument.  This of this like an = operator."
        "e.g. dc1.set_to(5), or dc1.set_to(dc2)"
        "This differs from set_to in that, if you were to destroy the argument immediately after, this is twice as efficient"
        "e.g. the two following operations are equivalent:"
        " "
        "1: dc1.set_to(dc2), dc2.set_to(0), dc2.destroy()"
        "2: dc1.become(dc2)"
        " "
        "but the second is twice as efficient as the first."
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Set To", var*self.offset_num)
                elif isinstance(var, DC):
                
                    if var*self.offset_num.max > self.max:
                        print ("You just tried to set a DC past its max value.")
                        sys.exit()
                        
                    max_power = ((var*self.offset_num.max).bit_length()-1)
                    out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), self+(2**i)]) )
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Set To", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    if isinstance(var, EUD_Remastered):
                    
                        #if var.max*self.offset_num > self.max:
                        #    print ("You just tried to set a DC past its max value.")
                        #    sys.exit()
                            
                        #max_power = ((var.max*self.offset_num).bit_length()-1)
                        max_power = 7
                        out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
                        for i in range(max_power, -1, -1):
                            out.append( Triggers.Trigger.T( C=[var>=(2**i)], A=[var-(2**i), self+(2**i)]) )
                        return out
                        
                    
                    else:
                        if var.max*self.offset_num > self.max:
                            print ("You just tried to set a DC past its max value.")
                            sys.exit()
                            
                        max_power = ((var.max*self.offset_num).bit_length()-1)
                        out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
                        for i in range(max_power, -1, -1):
                            out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), self+(2**i)]) )
                        return out
                
        else:
            return Triggers.Actions.memory_write(self.index, "Set To", self.base_address-(var*self.mult))
            
        
    def __add__(self, var):
        "Overloads the + operator, so you can write things like the following:"
        "dc1 + 2, which is equivalent to saying dc1 = dc1 + 2"
        "dc1 + dc2, which is equivalent to saying dc1 = dc1 + dc2"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Add", var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Add", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Triggers.Actions.memory_write(self.index, "Subtract", (var*self.mult))
            
    def add(self, var, player = None):
        "Alternative way of calling the + operator."
        return self + var
        
    def add_del(self, var):
        "Adds the value of the argument to the calling DC, and destroys the argument after."
        "Twice as efficient if you were to manually destroy the argument after."
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Add", var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), self+(2**i)]) )
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Add", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), self+(2**i)]) )
                    return out
                
        else:
            return Triggers.Actions.memory_write(self.index, "Subtract", (var*self.mult))
    
    def __sub__(self, var):
        "Overloads the - operator, so you can write things like the following:"
        "dc1 - 2, which is equivalent to saying dc1 = dc1 - 2"
        "dc1 - dc2, which is equivalent to saying dc1 = dc1 - dc2"
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Subtract", var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self-(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
                    temp.destroy()
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Subtract", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    temp = TempDC("Any")
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), temp+(2**i), self-(2**i)]) )
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var*self.offset_num+2**i]) )
                    temp.destroy()
                    return out
        else:
            return Triggers.Actions.memory_write(self.index, "Add", (var*self.mult))
            
    def sub(self, var, player = None):
        "Alternative way of calling the - operator."
        return self - var
        
    def sub_del(self, var):
        "Subtracts the value of the argument to the calling DC, and destroys the argument after."
        "Twice as efficient if you were to manually destroy the argument after."
        if(self.standard):
            if(not self.masked):
                if isinstance(var, int):
                    return Triggers.Actions.memory_write(self.index, "Subtract", var*self.offset_num)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), self-(2**i)]) )
                    return out
            else:
                if isinstance(var, int):
                    return Triggers.Actions.masked_memory_write(self.index, "Subtract", var*self.offset_num,self.bit_mask)
                elif isinstance(var, DC):
                    max_power = ((min(self.max, var*self.offset_num.max)).bit_length()-1)
                    out = []
                    for i in range(max_power, -1, -1):
                        out.append( Triggers.Trigger.T( C=[var*self.offset_num>=2**i], A=[var*self.offset_num-(2**i), self-(2**i)]) )
                    return out
        else:
            return Triggers.Actions.memory_write(self.index, "Add", (var*self.mult))
    
    
        
trigger_cycle = EUD_Remastered('0x006509a0',0)
fog_of_war = EUD_Remastered('0x0051CE98',0)      
current_player_pointer = EUD_Remastered(203155,0)      
        
        

        
        
        
        
        
        
        
        
        
        
        