# Location Class

from _globals import *
import Triggers

class Location:
    effects_location = None
    effects_player = None


    def __init__(self, name):
        if isinstance(name, str):
            self.name = name
        elif isinstance(name, int):
            self.name = "Location "+str(name)
            
    # Conditions
    def bring(self, player = "Current Player", unit = "Any unit", qmod = "At least", num = 1):
        return Triggers.Conditions.bring(player, unit, self.name, qmod, num)
        
    def bring_none(self, player = "Current Player", unit = "Any unit", qmod = "Exactly", num = 0):
        return Triggers.Conditions.bring(player, unit, self.name, qmod, num)
    
    # Actions
    def center_view(self):
        return Triggers.Actions.center_view(self.name)
    
    def create_unit(self, player, unit, num = 1, property = None):
        return Triggers.Actions.create_unit(player, unit, num, self.name, property)
        
    def give_unit(self, old_player, new_player, unit = "Any unit", num = "All"):
        return Triggers.Actions.give_unit(old_player, new_player, unit, num, self.name)
        
    def kill_unit(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.kill_unit_at_location(player, unit, num, self.name)
        
    def set_energy(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, unit, amount, num, self.name)
        
    def max_energy(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, unit, 100, num, self.name)
        
    def kill_energy(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, unit, 0, num, self.name)
        
    def set_hanger_count(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, unit, amount, num, self.name)
        
    def max_hanger_count(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, unit, 200, num, self.name)
        
    def kill_hanger_count(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, unit, 0, num, self.name)
        
    def set_hit_points(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_hit_points(player, unit, amount, num, self.name)
        
    def max_hit_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_hit_points(player, unit, 100, num, self.name)
        
    def set_shield_points(self, amount, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, unit, amount, num, self.name)
        
    def max_shield_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, unit, 100, num, self.name)
        
    def kill_shield_points(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, unit, 0, num, self.name)
        
    def set_resource_amount(self, amount, player = "Player 12", num = "All"):
        return Triggers.Actions.modify_unit_resource_amount(player, amount, num, self.name)    
        
    def move(self, player, unit = None, destination = "Anywhere"):
        if isinstance(player, Location):
            return Triggers.Actions.move_location("Player 12", "Map Revealer", player.name, self.name)
        return Triggers.Actions.move_location(player, unit, destination, self.name)
        
    def move_unit(self, player, destination, unit = "Any unit", num = "All"):
        return Triggers.Actions.move_unit(player, unit, num, self.name, destination)
        
    def order_attack_to(self, player, destination, unit = "Any unit"):
        return Triggers.Actions.order(player, unit, self.name, destination, "Attack")
        
    def order_move_to(self, player, destination, unit = "Any unit"):
        return Triggers.Actions.order(player, unit, self.name, destination, "Move")
        
    def order_patrol_to(self, player, destination, unit = "Any unit"):
        return Triggers.Actions.order(player, unit, self.name, destination, "Patrol")    
        
    def remove_unit(self, player = "Current Player", unit = "Any unit", num = "All"):
        return Triggers.Actions.remove_unit_at_location(player, unit, num, self.name)
        
    def ai_script(self, type):
        return Triggers.Actions.run_ai_script_at_location(type, self.name)
        
    def set_doodad_state(self, player = "Current Player", unit = "Any unit", state = "enabled"):
        return Triggers.Actions.set_doodad_state(player, unit, self.name, state)
        
    def set_invincibility(self, player = "Current Player", unit = "Any unit", state = "enabled"):
        return Triggers.Actions.set_invincibility(player, unit, self.name, state)    
        
    def enter_transport(self):
        return Triggers.Actions.run_ai_script_at_location("EnTr", self.name)
        
    def exit_transport(self):
        return Triggers.Actions.run_ai_script_at_location("ExTr", self.name)    
        
    def junk_yard_dog(self):
        return Triggers.Actions.run_ai_script_at_location("JYDg", self.name)
        
    def disruption_web(self):
        return Triggers.Actions.run_ai_script_at_location("CWHe", self.name)
        
    def recall(self):
        return Triggers.Actions.run_ai_script_at_location("ReHe", self.name)    
        
    def ping(self):
        return Triggers.Actions.minimap_ping(self.name)
        
    # Specials that require initialization from user before use
    def effect(self, unit):
        return [Location.effects_location.create_unit(Location.effects_player, unit), 
                Location.effects_location.move_unit(Location.effects_player, self, unit),
                Triggers.Actions.kill_unit(Location.effects_player, unit)]
                
    def down1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 1),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]
                
    def right1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 3),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]
                
    def up1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 5),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]            
                
    def left1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 7),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]            
                
    def downright1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 2),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]
                
    def upright1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 4),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]
                
    def upleft1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 6),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]            
                
    def downleft1_5(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Devourer", 8),
                location.create_unit(Location.movement_player2, "Zerg Devourer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Devourer"),
                self.move(Location.movement_player2, "Zerg Devourer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Devourer")]               
                
    def down1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player2, "Zerg Scourge", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]             
    
    def right1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player2, "Zerg Scourge", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]         

    def up1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player2, "Zerg Scourge", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]         

    def left1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player2, "Zerg Scourge", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]         

    def downright1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player2, "Protoss Observer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Protoss Observer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]             
    
    def upright1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player2, "Protoss Observer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Protoss Observer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]         

    def upleft1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player1, "Protoss Observer", 1),
                location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player2, "Protoss Observer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Protoss Observer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]               

    def downright1(self, location):
        return [location.create_unit(Location.movement_player1, "Zerg Scourge", 1),
                location.create_unit(Location.movement_player2, "Protoss Observer", 1),
                Triggers.Actions.remove_unit(Location.movement_player1, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player1, "Protoss Observer"),
                self.move(Location.movement_player2, "Protoss Observer"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Zerg Scourge"),
                Triggers.Actions.remove_unit(Location.movement_player2, "Protoss Observer")]         

    def down2(self, location):
        return [self.down1(location), self.down1(self)]

    def right2(self, location):
        return [self.right1(location), self.right1(self)]

    def up2(self, location):
        return [self.up1(location), self.up1(self)]

    def left2(self, location):
        return [self.left1(location), self.left1(self)]

    def downright2(self, location):
        return [self.downright1(location), self.downright1(self)]

    def downleft2(self, location):
        return [self.downleft1(location), self.downleft1(self)]

    def upleft2(self, location):
        return [self.upleft1(location), self.upleft1(self)]

    def upright2(self, location):
        return [self.upright1(location), self.upright1(self)]


    
                
                
def init_effects(player, location):
    Location.effects_player = player
    Location.effects_location = location
    return 
        
def init_movement(player1,player2):
    Location.movement_player1 = player1
    Location.movement_player2 = player2
    return
        
        
        
        
        
        
        
        
        
        
        
        
        
        