# Location Class

from _globals import *
import Triggers
from Classes.class_dc import DC
from Classes.class_tempdc import TempDC

resource_max = (2**31)-1

class Resource(DC):
    def __init__(self, name, player = "Current Player"):
        global resource_max
        self.name = name
        self.player = player
        self.max = resource_max
        
    # Conditions
    
    def __eq__(self, var):
        "overloads == operator, letting you do minerals == 7, or gas == dc2"
        if isinstance(var, int):
            return Triggers.Conditions.accumulate(self.player,"Exactly",var,self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i, self>=2**i], A=[var-(2**i), self-(2**i), temp+(2**i)]) )
            out.append( Triggers.Trigger.T( C=[self == 0, var == 0], A=[True] ) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i, self+2**i]) )
            temp.destroy()
            return out
    
    def __ge__(self, var):
        "overloads >= operator"
        if isinstance(var, int):
            if var == 0:
                return Triggers.Conditions.always()
            return Triggers.Conditions.accumulate(self.player,"At least",var,self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i, self>=2**i], A=[var-(2**i), self-(2**i), temp+(2**i)]) )
            out.append( Triggers.Trigger.T( C=[var == 0], A=[True] ) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i, self+2**i]) )
            temp.destroy()
            return out
    
    def __gt__(self, var):
        "overloads > operator"
        if isinstance(var, int):
            return Triggers.Conditions.accumulate(self.player,"At least",var+1,self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i, self>=2**i], A=[var-(2**i), self-(2**i), temp+(2**i)]) )
            out.append( Triggers.Trigger.T( C=[var == 0, self > 0], A=[True] ) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i, self+2**i]) )
            temp.destroy()
            return out
            
    def __le__(self, var):
        "overloads <= operator"
        if isinstance(var, int):
            if var == 0:
                return Triggers.Conditions.accumulate(self.player,"Exactly",0,self.name)
            return Triggers.Conditions.accumulate(self.player,"At most",var,self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i, self>=2**i], A=[var-(2**i), self-(2**i), temp+(2**i)]) )
            out.append( Triggers.Trigger.T( C=[self == 0], A=[True] ) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i, self+2**i]) )
            temp.destroy()
            return out
            
    def __lt__(self, var):
        "overloads < operator"
        if isinstance(var, int):
            if var == 0:
                return Triggers.Conditions.never()
            return self <= (var-1)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i, self>=2**i], A=[var-(2**i), self-(2**i), temp+(2**i)]) )
            out.append( Triggers.Trigger.T( C=[self == 0, var > 0], A=[True] ) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i, self+2**i]) )
            temp.destroy()
            return out
        
    #Actions
    def set_to(self, var):
        "Sets the DC to its argument.  This of this like an = operator."
        "e.g. dc1.set_to(5), or dc1.set_to(dc2)"
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Set to", var, self.name)
        elif isinstance(var, DC):
        
            if var.max > self.max:
                print ("You just tried to set a DC past its max value.")
                sys.exit()
        
            max_power = ((var.max).bit_length()-1)
            out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+(2**i)]) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
            temp.destroy()
            return out
            
    def become(self, var):
        "Sets the DC to its argument.  This of this like an = operator."
        "e.g. dc1.set_to(5), or dc1.set_to(dc2)"
        "This differs from set_to in that, if you were to destroy the argument immediately after, this is twice as efficient"
        "e.g. the two following operations are equivalent:"
        " "
        "1: dc1.set_to(dc2), dc2.set_to(0), dc2.destroy()"
        "2: dc1.become(dc2)"
        " "
        "but the second is twice as efficient as the first."
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Set to", var, self.name)
        elif isinstance(var, DC):
        
            if var.max > self.max:
                print ("You just tried to set a DC past its max value.")
                sys.exit()
                
            max_power = ((var.max).bit_length()-1)
            out = [Triggers.Trigger.T( A=[self.set_to(0)] )]
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), self+(2**i)]) )
            return out
            
        
    def __add__(self, var):
        "Overloads the + operator, so you can write things like the following:"
        "dc1 + 2, which is equivalent to saying dc1 = dc1 + 2"
        "dc1 + dc2, which is equivalent to saying dc1 = dc1 + dc2"
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Add", var, self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self+(2**i)]) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
            temp.destroy()
            return out
            
    def add(self, var):
        "Alternative way of calling the + operator."
        return self + var
        
    def add_del(self, var):
        "Adds the value of the argument to the calling DC, and destroys the argument after."
        "Twice as efficient if you were to manually destroy the argument after."
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Add", var, self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), self+(2**i)]) )
            return out
    
    def __sub__(self, var):
        "Overloads the - operator, so you can write things like the following:"
        "dc1 - 2, which is equivalent to saying dc1 = dc1 - 2"
        "dc1 - dc2, which is equivalent to saying dc1 = dc1 - dc2"
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Subtract", var, self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            temp = TempDC("Any")
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), temp+(2**i), self-(2**i)]) )
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[temp>=2**i], A=[temp-2**i, var+2**i]) )
            temp.destroy()
            return out
            
    def sub(self, var):
        "Alternative way of calling the - operator."
        return self - var
        
    def sub_del(self, var):
        "Subtracts the value of the argument to the calling DC, and destroys the argument after."
        "Twice as efficient if you were to manually destroy the argument after."
        if isinstance(var, int):
            return Triggers.Actions.set_resources(self.player, "Subtract", var, self.name)
        elif isinstance(var, DC):
            max_power = ((min(self.max, var.max)).bit_length()-1)
            out = []
            for i in range(max_power, -1, -1):
                out.append( Triggers.Trigger.T( C=[var>=2**i], A=[var-(2**i), self-(2**i)]) )
            return out
    
    
        
gas = Resource("gas")
ore = Resource("ore")
minerals = Resource("ore")
ore_and_gas = Resource("ore and gas")
minerals_and_gas = Resource("ore and gas")
        
        

        
        
        
        
        
        
        
        
        
        
        