# Location Class

from _globals import *
import Triggers
from Classes.class_location import Location


class Unit:
    def __init__(self, name, in_game_name = None):
        self.name = name
        self.in_game_name = in_game_name
        
    # Conditions
    def bring(self, player, location = "Anywhere", qmod = "At least", num = 1):
        return Triggers.Conditions.bring(player, self.name, location, qmod, num)
        
    def bring_none(self, player, location = "Anywhere"):
        return Triggers.Conditions.bring(player, self.name, location, "Exactly", 0)
        
    def command(self, player = "Current Player", qmod = "At least", num = 1):
        return Triggers.Conditions.command(player, self.name, qmod, num)
        
    def deaths(self, player = "Current Player", qmod = "At least", num = 1):
        return Triggers.Conditions.deaths(player, self.name, qmod, num)
        
    def overall_kills(self, num = 1, player = "Current Player", qmod = "At least"):
        return Triggers.Conditions.kill(player, self.unit, qmod, num)
        
    def kills(self, num = 1, player = "Current Player", qmod = "Exactly"):
        global unit_scores
        return Triggers.Conditions.score(player, "Kills", qmod, num*unit_scores[unit.name])
        
    # Actions
    def create(self, player, location = "Anywhere", num = 1, property = None):
        return Triggers.Actions.create_unit(player, self.name, num, location, property)
        
    def give(self, old_player, new_player, location = "Anywhere", num = "All"):
        return Triggers.Actions.give_unit(old_player, new_player, self.name, num, location)
        
    def kill(self, player = "Current Player", location = "Anywhere", num = "All"):
        if isinstance(location, Location):
            location = location.name
            
        if location == "Anywhere":
            return Triggers.Actions.kill_unit(player, self.name)
        return Triggers.Actions.kill_unit_at_location(player, self.name, num, location)
        
    def set_energy(self, amount, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, self.name, amount, num, location)
        
    def max_energy(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, self.name, 100, num, location)
        
    def kill_energy(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_energy(player, self.name, 0, num, location)
        
    def set_hit_points(self, amount, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_hit_points(player, self.name, amount, num, location)
        
    def max_hit_points(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_hit_points(player, self.name, 100, num, location)
        
    def set_shield_points(self, amount, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, self.name, amount, num, location)
        
    def max_shield_points(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, self.name, 100, num, location)
        
    def kill_shield_points(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_shield_points(player, self.name, 0, num, location)       
        
    def set_hanger_count(self, amount, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, self.name, amount, num, location)
        
    def max_hanger_count(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, self.name, 100, num, location)
        
    def kill_hanger_count(self, player = "Current Player", location = "Anywhere", num = "All"):
        return Triggers.Actions.modify_unit_hanger_count(player, self.name, 0, num, location)
        
    def center_location(self, player, location, location_of_unit = "Anywhere"):
        return Triggers.Actions.move_location(player, self.name, location_of_unit, location)
    
    def move(self, player, from_location, to_location, num = "All"):
        return Triggers.Actions.move_unit(player, self.name, num, from_location, to_location)
        
    def order_move(self, player, from_location, to_location):
        return Triggers.Actions.order(player, self.name, from_location, to_location, "Move")
        
    def order_attack(self, player, from_location, to_location):
        return Triggers.Actions.order(player, self.name, from_location, to_location, "Attack")    
        
    def order_patrol(self, player, from_location, to_location,):
        return Triggers.Actions.order(player, self.name, from_location, to_location, "Patrol")    
        
    def remove(self, player = "Current Player", location = "Anywhere", num = "All"):
        if isinstance(location, Location):
            location = location.name
            
        if location == "Anywhere":
            return Triggers.Actions.remove_unit(player, self.name)
        return Triggers.Actions.remove_unit_at_location(player, self.name, num, location)
        
    def set_deaths(self, num, player = "Current Player", mod = "Set to"):
        return Triggers.Actions.set_deaths(player, self.name, mod, num)
        
    def set_doodad_state(self, player = "All Players", location = "Anywhere", state = "enabled"):
        return Triggers.Actions.set_doodad_state(player, self.name, location, state)
        
    def set_invincibility(self, player = "All Players", location = "Anywhere", state = "enabled"):
        return Triggers.Actions.set_invincibility(player, self.name, location, state)     
        
        
        
        
        
        
        
        
        
        
        
        
        